@@ -85,10 +85,7 @@ struct _exsltDate {
 #define IS_TZO_CHAR(c)						\
 	((c == 0) || (c == 'Z') || (c == '+') || (c == '-'))
 
-#define TZO_SECS(tm)		(tm->tm_isdst > 0 ? 3600 : 0)
-
 #define VALID_YEAR(yr)          (yr != 0)
-/* months are stored as 0 - 11 */
 #define VALID_MONTH(mon)        ((mon >= 1) && (mon <= 12))
 /* VALID_DAY should only be used when month is unknown */
 #define VALID_DAY(day)          ((day >= 1) && (day <= 31))
@@ -104,7 +101,7 @@ static const int daysInMonth[12] =
 
 #define VALID_MDAY(dt)						\
 	(((dt->mon == 2) && !IS_LEAP(dt->year) && (dt->day <= 28)) ||\
-	 (dt->day == daysInMonth[dt->mon - 1]))
+	 (dt->day <= daysInMonth[dt->mon - 1]))
 
 #define VALID_DATE(dt)						\
 	(VALID_YEAR(dt->year) && VALID_MONTH(dt->mon) && VALID_MDAY(dt))
@@ -119,9 +116,14 @@ static const int daysInMonth[12] =
 
 static const int dayInYearByMonth[12] =
 	{ 0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334 };
+static const int dayInLeapYearByMonth[12] =
+	{ 0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335 };
 
 #define DAY_IN_YEAR(day, month, year)				\
-	(dayInYearByMonth[month - 1] + (IS_LEAP(year) ? 1 : 0))
+        ((IS_LEAP(year) ?					\
+                dayInLeapYearByMonth[month - 1] :		\
+                dayInYearByMonth[month - 1]) + day)
+
 #define DAY_IN_WEEK(yday, yr)					\
 	(((yr-1)+((yr-1)/4)-((yr-1)/100)+((yr-1)/400)+yday) % 7)
 
@@ -616,10 +618,12 @@ exsltDateFreeDate (exsltDatePtr date) {
 static exsltDatePtr
 exsltDateCurrent (void) {
     struct tm *localTm, *gmTm;
-    time_t tzSecs, secs;
+    time_t secs;
     exsltDatePtr ret;
 
     ret = exsltDateCreateDate();
+    if (ret == NULL)
+        return NULL;
 
     /* get current time */
     secs    = time(NULL);
@@ -630,19 +634,20 @@ exsltDateCurrent (void) {
     /* get real year, not years since 1900 */
     ret->year = localTm->tm_year + 1900;
 
-    ret->mon = localTm->tm_mon;
-    ret->day = localTm->tm_mday;
+    ret->mon  = localTm->tm_mon + 1;
+    ret->day  = localTm->tm_mday;
     ret->hour = localTm->tm_hour;
-    ret->min = localTm->tm_min;
+    ret->min  = localTm->tm_min;
 
     /* floating point seconds */
-    ret->sec         = (double) localTm->tm_sec;
+    ret->sec  = (double) localTm->tm_sec;
 
     /* determine the time zone offset from local to gm time */
     gmTm         = gmtime(&secs);
-    tzSecs       = mktime(gmTm) - secs;
     ret->tz_flag = 0;
-    ret->tzo = -(tzSecs - TZO_SECS(localTm)) / 60;
+    ret->tzo     = -(((ret->day - gmTm->tm_mday) * 1440) +
+                     ((ret->hour - gmTm->tm_hour) * 60) +
+                     (ret->min - gmTm->tm_min));
 
     return ret;
 }
@@ -1266,6 +1271,62 @@ exsltDateWeekInYear (const xmlChar *dateTime) {
     return (double) ret;
 }
 
+/**
+ * exsltDateWeekInMonth:
+ * @dateTime: a date/time string
+ *
+ * Implements the EXSLT - Dates and Times week-in-month() function
+ *    number date:week-in-month (string?)
+ * The date:week-in-month function returns the week in a month of a
+ * date as a number. If no argument is given, then the current local
+ * date/time, as returned by date:date-time is used the default
+ * argument. For the purposes of numbering, the first day of the month
+ * is in week 1 and new weeks begin on a Monday (so the first and last
+ * weeks in a month will often have less than 7 days in them).
+ * The date/time string specified as the argument is a right-truncated
+ * string in the format defined as the lexical representation of
+ * xs:dateTime in one of the formats defined in [XML Schema Part 2:
+ * Datatypes].  The permitted formats are as follows:
+ *  - xs:dateTime (CCYY-MM-DDThh:mm:ss)
+ *  - xs:date (CCYY-MM-DD)
+ * If the date/time string is not in one of these formats, then NaN is
+ * returned.
+ */
+static double
+exsltDateWeekInMonth (const xmlChar *dateTime) {
+    exsltDatePtr dt;
+    int fdiy, fdiw, ret;
+
+    if (dateTime == NULL) {
+#ifdef WITH_TIME
+	dt = exsltDateCurrent();
+	if (dt == NULL)
+#endif
+	    return xmlXPathNAN;
+    } else {
+	dt = exsltDateParse(dateTime);
+	if (dt == NULL)
+	    return xmlXPathNAN;
+	if ((dt->type != XS_DATETIME) && (dt->type != XS_DATE)) {
+	    exsltDateFreeDate(dt);
+	    return xmlXPathNAN;
+	}
+    }
+
+    fdiy = DAY_IN_YEAR(1, dt->mon, dt->year);
+    /* 0=Mon, 1=Tue, etc. */
+    fdiw = DAY_IN_WEEK(fdiy, dt->year);
+
+    /* adjust the fdiw value so that Sunday is last-day-in-week */
+    fdiw = (fdiw != 0 ? (fdiw - 1) : 6);
+
+    ret = ((dt->day + fdiw) / 7) + 1;
+
+    exsltDateFreeDate(dt);
+
+    return (double) ret;
+}
+
 /**
  * exsltDateDayInYear:
  * @dateTime: a date/time string
@@ -1412,8 +1473,8 @@ exsltDateDayOfWeekInMonth (const xmlChar *dateTime) {
  * exsltDateDayInWeek:
  * @dateTime: a date/time string
  *
- * Implements the EXSLT - Dates and Times day-in-month() function:
- *    number date:day-in-month (string?)
+ * Implements the EXSLT - Dates and Times day-in-week() function:
+ *    number date:day-in-week (string?)
  * Returns the day of the week given in a date as a number.  If no
  * argument is given, then the current local date/time, as returned by
  * date:date-time is used the default argument.
@@ -1963,6 +2024,15 @@ exsltDateMonthAbbreviationFunction (xmlXPathParserContextPtr ctxt,
  */
 X_IN_Y(Week,Year)
 
+/**
+ * exsltDateWeekInMonthFunction:
+ * @ctxt: an XPath parser context
+ * @nargs : the number of arguments
+ *
+ * Wraps #exsltDateWeekInMonthYear for use by the XPath engine
+ */
+X_IN_Y(Week,Month)
+
 /**
  * exsltDateDayInYearFunction:
  * @ctxt: an XPath parser context
@@ -2138,6 +2208,9 @@ exsltDateRegister(void)
     xsltRegisterExtModuleFunction((const xmlChar *) "week-in-year",
 			  (const xmlChar *) EXSLT_DATE_NAMESPACE,
 			  exsltDateWeekInYearFunction);
+    xsltRegisterExtModuleFunction((const xmlChar *) "week-in-month",
+			  (const xmlChar *) EXSLT_DATE_NAMESPACE,
+			  exsltDateWeekInMonthFunction);
     xsltRegisterExtModuleFunction((const xmlChar *) "day-in-year",
 			  (const xmlChar *) EXSLT_DATE_NAMESPACE,
 			  exsltDateDayInYearFunction);
