@@ -298,8 +298,17 @@ xsltPrintErrorContext(xsltTransformContextPtr ctxt,
 
 	    file = doc->URL;
 	} else {
+	    /*
+	     * Try to find contextual informations to report
+	     */
 	    if (node->type == XML_ELEMENT_NODE) {
 		line = (int) node->content;
+	    } else if ((node->prev != NULL) &&
+		       (node->prev->type == XML_ELEMENT_NODE)) {
+		line = (int) node->prev->content;
+	    } else if ((node->parent != NULL) &&
+		       (node->parent->type == XML_ELEMENT_NODE)) {
+		line = (int) node->parent->content;
 	    }
 	    if ((node->doc != NULL) && (node->doc->URL != NULL))
 		file = node->doc->URL;
